﻿Option Strict Off
Public Class CourseForm

    Private Sub Course_mainBindingNavigatorSaveItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Course_mainBindingNavigatorSaveItem.Click, MyBase.FormClosing
        Me.Validate()
        ' 入力エラーがある場合は保存しない
        If HandleInputError() = False Then
            ' フォームを閉じる前の場合、閉じる処理をキャンセルする
            If TypeName(e) = "FormClosingEventArgs" Then
                DirectCast(e, FormClosingEventArgs).Cancel = True
            End If
            Exit Sub
        End If
        ' 入力中のレコードの編集を確定させる
        EndEditRecord()
        ' レコードをテーブルに保存する
        Me.TableAdapterManager.UpdateAll(Me.GolfDataSet)
    End Sub

    Private Sub CourseForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.Course_mainTableAdapter.Fill(Me.GolfDataSet.course_main)
        Me.Course_subTableAdapter.Fill(Me.GolfDataSet.course_sub)
        ' ホールの番号を設定する
        gridCourseData.Rows.Add(18)
        For I = 0 To 17
            gridCourseData.Rows(I).Cells(0).Value = I + 1
        Next
        ' レコードがあれば、1番目のコースのサブレコードの情報を表示する
        If Course_mainBindingSource.Count > 0 Then
            ShowRecord()
        End If
    End Sub

    Private Sub BindingNavigatorAddNewItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorAddNewItem.Click
        Dim dv As New DataView, newId As Integer, I As Integer

        ' 入力エラーのチェック
        If HandleInputError() = False Then
            Exit Sub
        End If
        ' 入力中のレコードの編集を確定させる
        EndEditRecord()
        ' cource_mainテーブルをDataViewにコピーし、id列の値が大きい順に並べ替える
        dv.Table = GolfDataSet.course_main
        dv.Sort = "id desc"
        If dv.Count = 0 Then
            ' cource_mainテーブルにレコードがなければ、新しいレコードのidを1にする
            newId = 1
        Else
            ' cource_mainテーブルにレコードがあれば、
            ' 新しいレコードのidを、最大のidの次の番号にする
            newId = dv.Item(0)("id") + 1
        End If
        ' レコードを新規作成する
        Course_mainBindingSource.AddNew()
        ' idのテキストボックスに新しいレコードのidを設定する
        IdTextBox.Text = newId
        ' 各ホールのヤードとパーの初期値を設定する
        For I = 0 To 17
            gridCourseData.Rows(I).Cells(1).Value = 400
            gridCourseData.Rows(I).Cells(2).Value = 4
        Next
        ' コース全体のパーを72にする
        txtPar.Text = "72"
    End Sub

    Private Sub BindingNavigatorMoveFirstItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMoveFirstItem.Click
        ' 入力エラーのチェック
        If HandleInputError() = False Then
            Exit Sub
        End If
        ' レコードの編集を確定する
        EndEditRecord()
        ' 先頭のレコードに移動する
        Course_mainBindingSource.MoveFirst()
        ' 移動先レコードのコースサブ部分を表示する
        ShowRecord()
    End Sub

    Private Sub BindingNavigatorMovePreviousItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMovePreviousItem.Click
        ' 入力エラーのチェック
        If HandleInputError() = False Then
            Exit Sub
        End If
        ' レコードの編集を確定する
        EndEditRecord()
        ' 先頭のレコードに移動する
        Course_mainBindingSource.MovePrevious()
        ' 移動先レコードのコースサブ部分を表示する
        ShowRecord()
    End Sub

    Private Sub BindingNavigatorMoveNextItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMoveNextItem.Click
        ' 入力エラーのチェック
        If HandleInputError() = False Then
            Exit Sub
        End If
        ' レコードの編集を確定する
        EndEditRecord()
        ' 先頭のレコードに移動する
        Course_mainBindingSource.MoveNext()
        ' 移動先レコードのコースサブ部分を表示する
        ShowRecord()
    End Sub

    Private Sub BindingNavigatorMoveLastItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMoveLastItem.Click
        ' 入力エラーのチェック
        If HandleInputError() = False Then
            Exit Sub
        End If
        ' レコードの編集を確定する
        EndEditRecord()
        ' 先頭のレコードに移動する
        Course_mainBindingSource.MoveLast()
        ' 移動先レコードのコースサブ部分を表示する
        ShowRecord()
    End Sub

    Private Sub BindingNavigatorDeleteItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorDeleteItem.Click
        Dim dv As New DataView, r As DataRowView, courseId As Integer, I As Integer

        ' レコードがなければ何もしない
        If Course_mainBindingSource.Count = 0 Then
            Exit Sub
        End If
        ' 削除してよいかどうかを尋ねる
        If MsgBox("このコースを削除してもよろしいですか", MsgBoxStyle.YesNo + MsgBoxStyle.DefaultButton2) = MsgBoxResult.Yes Then
            ' サブのレコードを削除する
            courseId = IdTextBox.Text
            r = Course_mainBindingSource.Current
            If r.IsNew = False Then
                dv.Table = GolfDataSet.course_sub
                dv.RowFilter = "course_id = " & courseId
                For I = dv.Count - 1 To 0 Step -1
                    dv.Item(I).Delete()
                Next
            End If
            ' メインのレコードを削除する
            Course_mainBindingSource.RemoveCurrent()
            If Course_mainBindingSource.Count > 0 Then
                ' レコードがあれば、移動先レコードのコースサブ部分を表示する
                ShowRecord()
            Else
                ' レコードがなければ、DataGridViewの「ヤード」「パー」の列を空にする
                For I = 0 To 17
                    gridCourseData.Rows(I).Cells(1).Value = ""
                    gridCourseData.Rows(I).Cells(2).Value = ""
                Next
                ' コース全体のパーをクリアする
                txtPar.Text = ""
            End If
        End If
    End Sub

    Private Sub gridCourseData_CellValueChanged(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) Handles gridCourseData.CellValueChanged
        If e.RowIndex >= 0 And e.ColumnIndex = 1 Then
            ShowPar()
        End If
    End Sub

    Sub ShowRecord()
        Dim dv As New DataView, r As DataRowView, courseId As Integer, I As Integer

        ' ホールのデータを読み込み、ホール番号順に並べ替える
        courseId = IdTextBox.Text
        dv.Table = GolfDataSet.course_sub
        dv.RowFilter = "course_id = " & courseId
        dv.Sort = "hole_no"
        ' 各ホールのヤードとパーをDataGridViewに表示する
        For I = 0 To 17
            r = dv.Item(I)
            gridCourseData.Rows(I).Cells(1).Value = r("yard")
            gridCourseData.Rows(I).Cells(2).Value = r("par")
        Next
        ' ホール全体のパーを計算して表示する
        ShowPar()
    End Sub

    Sub ShowPar()
        Dim par As Integer

        ' ホール全体のパーを計算する
        For I = 0 To 17
            par += Val(gridCourseData.Rows(I).Cells(2).Value)
        Next
        ' ホール全体のパーを表示する
        txtPar.Text = par
    End Sub

    Function HandleInputError() As Boolean
        ' レコードがなければエラーチェックしない
        If Course_mainBindingSource.Count = 0 Then
            Return True
        End If
        ' コース名が入力されていなければエラーにする
        If NameTextBox.Text = "" Then
            MsgBox("コース名を入力してください")
            Return False
        End If
        Return True
    End Function

    Sub EndEditRecord()
        Dim dv As New DataView, r As DataRowView, courseId As Integer, I As Integer
        Dim isNew As Boolean

        ' レコードを編集していない状態なら何もしない
        If Course_mainBindingSource.Current Is Nothing Then
            Return
        End If
        ' 現在のレコードが新規レコードかどうかを得る
        r = Course_mainBindingSource.Current
        isNew = r.IsNew
        ' コースメインのレコードの編集を確定する
        Course_mainBindingSource.EndEdit()
        ' コースのIDを得る
        courseId = IdTextBox.Text
        ' DhataViewにコースサブテーブルを結びつける
        dv.Table = GolfDataSet.course_sub
        If isNew Then
            ' 新規レコードの場合は、サブテーブルにレコードを追加する
            For I = 0 To 17
                r = dv.AddNew()
                r.BeginEdit()
                r("course_id") = courseId
                r("hole_no") = I + 1
                r("yard") = Val(gridCourseData.Rows(I).Cells(1).Value)
                r("par") = Val(gridCourseData.Rows(I).Cells(2).Value)
                r.EndEdit()
            Next
        Else
            ' 既存のレコードの場合は、サブテーブルのレコードを変更する
            dv.RowFilter = "course_id = " & courseId
            dv.Sort = "hole_no"
            For I = 0 To 17
                r = dv.Item(I)
                r.BeginEdit()
                r("yard") = Val(gridCourseData.Rows(I).Cells(1).Value)
                r("par") = Val(gridCourseData.Rows(I).Cells(2).Value)
                r.EndEdit()
            Next
        End If
    End Sub
End Class