﻿Option Strict Off
Public Class RoundForm

    Private Sub Round_mainBindingNavigatorSaveItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Round_mainBindingNavigatorSaveItem.Click, MyBase.FormClosing
        Me.Validate()
        ' 入力エラーがある場合は保存しない
        If HandleInputError() = False Then
            ' フォームを閉じる前の場合、閉じる処理をキャンセルする
            If TypeName(e) = "FormClosingEventArgs" Then
                DirectCast(e, FormClosingEventArgs).Cancel = True
            End If
            Exit Sub
        End If
        ' 入力中のレコードの編集を確定させる
        EndEditRecord()
        ' レコードをテーブルに保存する
        Me.TableAdapterManager.UpdateAll(Me.GolfDataSet)
    End Sub

    Private Sub RoundForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' テーブルからデータセットにレコードを読み込む
        Me.Course_mainTableAdapter.Fill(Me.GolfDataSet.course_main)
        Me.Course_subTableAdapter.Fill(Me.GolfDataSet.course_sub)
        Me.Round_mainTableAdapter.Fill(Me.GolfDataSet.round_main)
        Me.Round_subTableAdapter.Fill(Me.GolfDataSet.round_sub)
        ' ホールの番号を設定する
        gridRoundData.Rows.Add(18)
        For I = 0 To 17
            gridRoundData.Rows(I).Cells(0).Value = I + 1
        Next
        ' レコードがあれば、1番目のラウンドのサブレコードの情報を表示する
        If Round_mainBindingSource.Count > 0 Then
            ShowRecord()
            ShowCourseData()
        End If
    End Sub

    Private Sub BindingNavigatorAddNewItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorAddNewItem.Click
        Dim dv As New DataView, newId As Integer, I As Integer

        ' 入力エラーのチェック
        If HandleInputError() = False Then
            Exit Sub
        End If
        ' 入力中のレコードの編集を確定させる
        EndEditRecord()
        ' round_mainテーブルをDataViewにコピーし、id列の値が大きい順に並べ替える
        dv.Table = GolfDataSet.round_main
        dv.Sort = "id desc"
        If dv.Count = 0 Then
            ' cource_mainテーブルにレコードがなければ、新しいレコードのidを1にする
            newId = 1
        Else
            ' cource_mainテーブルにレコードがあれば、
            ' 新しいレコードのidを、最大のidの次の番号にする
            newId = dv.Item(0)("id") + 1
        End If
        ' レコードを新規作成する
        Round_mainBindingSource.AddNew()
        ' idのテキストボックスに新しいレコードのidを設定する
        IdTextBox.Text = newId
        ' 日付のDateTimePickerに現在の日時を設定する
        DateDateTimePicker.Value = Now()
        ' ハンディキャップを36に設定する
        HandicapTextBox.Text = "36"
        ' 各ホールのパー／ヤード／スコアを0にクリアする
        For I = 0 To 17
            gridRoundData.Rows(I).Cells(1).Value = 0
            gridRoundData.Rows(I).Cells(2).Value = 0
            gridRoundData.Rows(I).Cells(3).Value = 0
        Next
        ' 全体のスコアをクリアする
        txtPar.Text = "0"
        txtGross.Text = "0"
        txtNet.Text = "0"
    End Sub

    Sub EndEditRecord()
        Dim dv As New DataView, r As DataRowView, roundId As Integer, I As Integer
        Dim isNew As Boolean

        ' レコードを編集していない状態なら何もしない
        If Round_mainBindingSource.Current Is Nothing Then
            Return
        End If
        ' 現在のレコードが新規レコードかどうかを得る
        r = Round_mainBindingSource.Current
        isNew = r.IsNew
        ' メインのレコードの編集を確定する
        Round_mainBindingSource.EndEdit()
        ' メインのIDを得る
        roundId = IdTextBox.Text
        ' DataViewにサブテーブルを結びつける
        dv.Table = GolfDataSet.round_sub
        If isNew Then
            ' 新規レコードの場合は、サブテーブルにレコードを追加する
            For I = 0 To 17
                r = dv.AddNew
                r.BeginEdit()
                r("round_id") = roundId
                r("hole_no") = I + 1
                r("score") = gridRoundData.Rows(I).Cells(3).Value
                r.EndEdit()
            Next
        Else
            ' 既存のレコードの場合は、サブテーブルのレコードを変更する
            dv.RowFilter = "round_id = " & roundId
            dv.Sort = "hole_no"
            For I = 0 To 17
                r = dv.Item(I)
                r.BeginEdit()
                r("score") = gridRoundData.Rows(I).Cells(3).Value
                r.EndEdit()
            Next
        End If
    End Sub

    Function HandleInputError() As Boolean
        ' レコードがなければエラーチェックをしない
        If Round_mainBindingSource.Count = 0 Then
            Return True
        End If
        ' コースが選ばれていなければエラーとする
        If Course_idComboBox.SelectedValue = 0 Then
            MsgBox("コースが選択されていません。")
            Course_idComboBox.Focus()
            Return False
        End If
        Return True
    End Function

    Private Sub BindingNavigatorMoveFirstItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMoveFirstItem.Click
        ' 入力エラーのチェック
        If HandleInputError() = False Then
            Exit Sub
        End If
        ' レコードの編集を確定する
        EndEditRecord()
        ' 先頭のレコードに移動する
        Round_mainBindingSource.MoveFirst()
        ' 移動先レコードのサブ部分を表示する
        ShowRecord()
    End Sub

    Private Sub BindingNavigatorMovePreviousItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMovePreviousItem.Click
        ' 入力エラーのチェック
        If HandleInputError() = False Then
            Exit Sub
        End If
        ' レコードの編集を確定する
        EndEditRecord()
        ' 先頭のレコードに移動する
        Round_mainBindingSource.MovePrevious()
        ' 移動先レコードのサブ部分を表示する
        ShowRecord()
    End Sub

    Private Sub BindingNavigatorMoveNextItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMoveNextItem.Click
        ' 入力エラーのチェック
        If HandleInputError() = False Then
            Exit Sub
        End If
        ' レコードの編集を確定する
        EndEditRecord()
        ' 先頭のレコードに移動する
        Round_mainBindingSource.MoveNext()
        ' 移動先レコードのサブ部分を表示する
        ShowRecord()
    End Sub

    Private Sub BindingNavigatorMoveLastItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMoveLastItem.Click
        ' 入力エラーのチェック
        If HandleInputError() = False Then
            Exit Sub
        End If
        ' レコードの編集を確定する
        EndEditRecord()
        ' 先頭のレコードに移動する
        Round_mainBindingSource.MoveLast()
        ' 移動先レコードのサブ部分を表示する
        ShowRecord()
    End Sub

    Sub ShowRecord()
        Dim dv As New DataView, r As DataRowView, roundId As Integer, I As Integer

        ' ラウンドIDを得る
        roundId = IdTextBox.Text
        ' このラウンドのサブのデータを得る
        dv.Table = GolfDataSet.round_sub
        dv.RowFilter = "round_id = " & roundId
        dv.Sort = "hole_no"
        ' 各ホールのスコアをDataGridViewに表示する
        For I = 0 To 17
            r = dv.Item(I)
            gridRoundData.Rows(I).Cells(3).Value = r("score")
        Next
        ShowScore()
    End Sub

    Sub ShowScore()
        Dim I, par, gross As Integer
        ' パーとスコアの列の合計を求める
        For I = 0 To 17
            par += gridRoundData.Rows(I).Cells(2).Value
            gross += gridRoundData.Rows(I).Cells(3).Value
        Next
        ' コース全体のパーを表示する
        txtPar.Text = par
        ' グロスのスコアを表示する
        txtGross.Text = gross
        ' グロスからハンディを引いてネットのスコアを表示する
        txtNet.Text = gross - Val(HandicapTextBox.Text)
    End Sub

    Private Sub BindingNavigatorDeleteItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorDeleteItem.Click
        Dim dv As New DataView, r As DataRowView, roundId As Integer, I As Integer

        ' 削除してよいかどうかを尋ねる
        If MsgBox("このラウンドデータを削除してもよろしいですか", MsgBoxStyle.YesNo + MsgBoxStyle.DefaultButton2) = MsgBoxResult.Yes Then
            ' サブのレコードを削除する
            roundId = IdTextBox.Text
            r = Round_mainBindingSource.Current()
            If r.IsNew = False Then
                dv.Table = GolfDataSet.round_sub
                dv.RowFilter = "round_id = " & roundId
                For I = dv.Count - 1 To 0 Step -1
                    dv.Item(I).Delete()
                Next
            End If
            ' メインのレコードを削除する
            Round_mainBindingSource.RemoveCurrent()
            If Round_mainBindingSource.Count > 0 Then
                ' レコードがあれば、移動先レコードのサブ部分を表示する
                ShowRecord()
            Else
                ' レコードがなければ、DataGridViewの「ヤード」「パー」「スコア」の列を空にする()
                For I = 0 To 17
                    gridRoundData.Rows(I).Cells(1).Value = 0
                    gridRoundData.Rows(I).Cells(2).Value = 0
                    gridRoundData.Rows(I).Cells(3).Value = 0
                Next
                ' コース全体のパーをクリアする
                txtPar.Text = "0"
                txtGross.Text = "0"
                txtNet.Text = "0"
            End If
        End If
    End Sub

    Private Sub Course_idComboBox_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Course_idComboBox.SelectedIndexChanged
        ShowCourseData()
    End Sub

    Sub ShowCourseData()
        Dim courseId As Integer, I As Integer
        Dim dv As New DataView, r As DataRowView

        ' コースのIDを得る
        courseId = Course_idComboBox.SelectedValue
        If courseId > 0 Then
            ' IDで示されたコースの各ホールのレコードを読み込む
            dv.Table = GolfDataSet.course_sub
            dv.RowFilter = "course_id = " & courseId
            dv.Sort = "hole_no"
            ' 各ホールのヤードとパーをDataGridViewに表示する
            For I = 0 To 17
                r = dv.Item(I)
                gridRoundData.Rows(I).Cells(1).Value = r("yard")
                gridRoundData.Rows(I).Cells(2).Value = r("par")
            Next
            ShowScore()
        End If
    End Sub

    Private Sub gridRoundData_CellValueChanged(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) Handles gridRoundData.CellValueChanged
        If e.RowIndex >= 0 And e.ColumnIndex = 3 Then
            ShowScore()
        End If
    End Sub
End Class