﻿Public Class OrderReport
    Private Declare Function GetDeviceCaps Lib "gdi32" _
           (ByVal hdc As IntPtr, ByVal nIndex As Integer) As Integer
    Private Const LOGPIXELSX = 88
    Private Const LOGPIXELSY = 90
    Private Const PHYSICALOFFSETX = 112
    Private Const PHYSICALOFFSETY = 113
    Dim isInPrinting As Boolean = False
    Dim dt_main As New OrderDataSet.OrderReportMainDataTable

    Private Sub OrderReport_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' DataTableに伝票メイン用のレコードを読み込む
        Dim ta_main As New OrderDataSetTableAdapters.OrderReportMainTableAdapter
        ta_main.Fill(dt_main)
        ' BindingNavigatorにレコード数とレコード番号を表示する
        BindingNavigatorCountItem.Text = "/" & dt_main.Count
        BindingNavigatorPositionItem.Text = 1
    End Sub

    Sub DrawReport(ByVal g As Graphics)
        Dim dt_sub As New OrderDataSet.OrderReportSubDataTable
        Dim ta_sub As New OrderDataSetTableAdapters.OrderReportSubTableAdapter
        Dim dr_main As DataRow, dr_sub As DataRow
        Dim order_date As Date, y As Integer, i As Integer, total As Integer

        ' 伝票メインのデータを得る
        dr_main = dt_main.Rows(BindingNavigatorPositionItem.Text - 1)
        order_date = dr_main("date")
        ' 伝票IDからそれに対応する伝票サブのDataTableを読み込む
        ta_sub.Fill(dt_sub, dr_main("id"))
        ' 描画サーフェイスの単位をミリメートルにする
        g.PageUnit = GraphicsUnit.Millimeter
        ' プリンタの印刷不可能範囲の分だけ、原点の位置をずらす
        If isInPrinting Then
            Dim hDC As IntPtr
            hDC = g.GetHdc()
            Dim dpiX = GetDeviceCaps(hDC, LOGPIXELSX)
            Dim dpiY = GetDeviceCaps(hDC, LOGPIXELSY)
            Dim marginX = GetDeviceCaps(hDC, PHYSICALOFFSETX)
            Dim marginY = GetDeviceCaps(hDC, PHYSICALOFFSETY)
            g.ReleaseHdc()
            g.TranslateTransform(-marginX / dpiX * 25.4, -marginY / dpiY * 25.4, Drawing2D.MatrixOrder.Append)
        End If
        ' 描画に必要なオブジェクトを作成する
        Dim blackSolidPen As New Pen(Color.Black, 0.1)
        Dim blackDotPen As New Pen(Color.Black, 0.1)
        blackDotPen.DashStyle = Drawing2D.DashStyle.Dot
        Dim rect As RectangleF
        Dim strForm As New StringFormat
        Dim gFont24 As New Font("ＭＳ Ｐゴシック", 24)
        Dim gFont16 As New Font("ＭＳ Ｐゴシック", 16)
        Dim mFont10 As New Font("ＭＳ Ｐ明朝", 10)
        Dim mFont12 As New Font("ＭＳ Ｐ明朝", 12)
        ' 「納品書」の文字と下線を表示
        strForm.Alignment = StringAlignment.Center
        strForm.LineAlignment = StringAlignment.Far
        rect = New RectangleF(0, 10, 210, 10)
        g.DrawString("納　品　書", gFont24, Brushes.Black, rect, strForm)
        g.DrawLine(blackSolidPen, 80, 20, 130, 20)
        ' 日付を表示
        strForm.Alignment = StringAlignment.Far
        rect = New RectangleF(150, 20, 50, 5)
        g.DrawString(Format(order_date, "yyyy年M月d日"), mFont10, Brushes.Black, rect, strForm)
        ' 自社名を表示
        rect = New RectangleF(150, 25, 50, 5)
        g.DrawString("○○文具卸株式会社", mFont10, Brushes.Black, rect, strForm)
        ' 顧客の住所／電話番号／名前を表示
        strForm.Alignment = StringAlignment.Near
        rect = New Rectangle(10, 30, 190, 5)
        g.DrawString(dr_main("address"), mFont10, Brushes.Black, rect, strForm)
        rect = New Rectangle(10, 35, 190, 5)
        g.DrawString(Trim(dr_main("phone")), mFont10, Brushes.Black, rect, strForm)
        rect = New Rectangle(10, 40, 190, 5)
        g.DrawString(dr_main("name") & "　御中", mFont10, Brushes.Black, rect, strForm)
        ' あいさつ文を表示
        rect = New Rectangle(10, 50, 190, 5)
        g.DrawString("毎度お買い上げいただきまことにありがとうございます。", mFont12, Brushes.Black, rect, strForm)
        rect = New Rectangle(10, 57, 190, 5)
        g.DrawString("下記の通り納品いたしましたのでご査収ください。", mFont12, Brushes.Black, rect, strForm)
        ' 商品一覧の先頭の行を表示
        g.FillRectangle(Brushes.DarkGray, New RectangleF(10, 70, 190, 10))
        ' 商品一覧の表に1行おきに網掛けを行う
        For y = 87 To 199 Step 14
            g.FillRectangle(Brushes.LightGray, New Rectangle(10, y, 190, 7))
        Next
        ' 商品一覧の最後の行を表示
        g.FillRectangle(Brushes.DarkGray, New RectangleF(10, 213, 190, 10))
        g.DrawLine(blackDotPen, 100, 70, 100, 213)
        g.DrawLine(blackDotPen, 130, 70, 130, 213)
        g.DrawLine(blackDotPen, 160, 70, 160, 223)
        ' 商品一覧の先頭の行に「品名」等の見出しを表示
        strForm.Alignment = StringAlignment.Center
        strForm.LineAlignment = StringAlignment.Center
        rect = New Rectangle(10, 70, 90, 10)
        g.DrawString("品名", gFont16, Brushes.Black, rect, strForm)
        rect = New Rectangle(100, 70, 30, 10)
        g.DrawString("単価", gFont16, Brushes.Black, rect, strForm)
        rect = New Rectangle(130, 70, 30, 10)
        g.DrawString("個数", gFont16, Brushes.Black, rect, strForm)
        rect = New Rectangle(160, 70, 40, 10)
        g.DrawString("金額", gFont16, Brushes.Black, rect, strForm)
        rect = New Rectangle(10, 213, 150, 10)
        g.DrawString("合計", gFont16, Brushes.Black, rect, strForm)
        ' 各商品の情報を表示
        For i = 0 To dt_sub.Count - 1
            y = 80 + i * 7
            dr_sub = dt_sub.Rows(i)
            strForm.Alignment = StringAlignment.Near
            rect = New Rectangle(10, y, 90, 7)
            g.DrawString(dr_sub("name"), mFont12, Brushes.Black, rect, strForm)
            strForm.Alignment = StringAlignment.Far
            rect = New Rectangle(100, y, 29, 7)
            g.DrawString(Format(dr_sub("price"), "\\#,##0"), mFont12, Brushes.Black, rect, strForm)
            rect = New Rectangle(130, y, 29, 7)
            g.DrawString(Format(dr_sub("count"), "#,##0"), mFont12, Brushes.Black, rect, strForm)
            rect = New Rectangle(160, y, 39, 7)
            g.DrawString(Format(dr_sub("subtotal"), "\\#,##0"), mFont12, Brushes.Black, rect, strForm)
            total = total + dr_sub("subtotal")
        Next
        ' 合計を表示
        rect = New Rectangle(160, 213, 39, 10)
        g.DrawString(Format(total, "\\#,##0"), gFont16, Brushes.Black, rect, strForm)
    End Sub

    Private Sub panelReport_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles panelReport.Paint
        ' 納品書を描画する
        DrawReport(e.Graphics)
    End Sub

    Private Sub BindingNavigatorMovePreviousItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMovePreviousItem.Click
        ' 先頭のレコードでなければ、前のレコードに移動()
        If BindingNavigatorPositionItem.Text > 1 Then
            BindingNavigatorPositionItem.Text -= 1
            panelReport.Refresh()
        End If
    End Sub

    Private Sub BindingNavigatorMoveNextItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMoveNextItem.Click
        ' 最後のレコードでなければ、次のレコードに移動
        If BindingNavigatorPositionItem.Text < dt_main.Count Then
            BindingNavigatorPositionItem.Text += 1
            panelReport.Refresh()
        End If
    End Sub

    Private Sub BindingNavigatorMoveFirstItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMoveFirstItem.Click
        ' 先頭のレコードに移動
        BindingNavigatorPositionItem.Text = 1
        panelReport.Refresh()
    End Sub

    Private Sub BindingNavigatorMoveLastItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMoveLastItem.Click
        ' 最後のレコードに移動
        BindingNavigatorPositionItem.Text = dt_main.Count
        panelReport.Refresh()
    End Sub

    Private Sub BindingNavigatorPrint_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorPrint.Click
        ' 「印刷」のダイアログボックスを開く
        PrintDialog1.Document = PrintDocument1
        If PrintDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then
            ' 現在表示中の納品書を印刷する
            isInPrinting = True
            PrintDocument1.Print()
            isInPrinting = False
        End If
    End Sub

    Private Sub PrintDocument1_PrintPage(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs) Handles PrintDocument1.PrintPage
        DrawReport(e.Graphics)
    End Sub

    Private Sub BindingNavigatorPrintPreview_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorPrintPreview.Click
        ' プレビューするPrintDocumentを指定する
        PrintPreviewDialog1.Document = PrintDocument1
        ' プレビューのダイアログボックスのサイズを決める
        PrintPreviewDialog1.Width = 480
        PrintPreviewDialog1.Height = 600
        ' プレビューのダイアログボックスを開く
        PrintPreviewDialog1.ShowDialog()
    End Sub
End Class