﻿Public Class OrderForm

    Private Sub Order_mainBindingNavigatorSaveItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Order_mainBindingNavigatorSaveItem.Click, MyBase.FormClosing
        Me.Validate()
        ' 入力エラーがあるときは何もしない
        If HandleInputError() = False Then
            ' フォームを閉じる前の場合、閉じる処理をキャンセルする
            If TypeName(e) = "FormClosingEventArgs" Then
                DirectCast(e, FormClosingEventArgs).Cancel = True
            End If
            Return
        End If
        ' 伝票サブ部分の編集を確定する
        EndEditSub()
        ' DataSetの内容をテーブルに保存する
        Me.TableAdapterManager.UpdateAll(Me.OrderDataSet)
    End Sub

    Private Sub OrderForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.ItemTableAdapter.Fill(Me.OrderDataSet.item)
        Me.CustomerTableAdapter.Fill(Me.OrderDataSet.customer)
        Me.Order_mainTableAdapter.Fill(Me.OrderDataSet.order_main)
        Me.Order_subTableAdapter.Fill(Me.OrderDataSet.order_sub)
        CalcTotal()
    End Sub

    Private Sub Customer_idComboBox_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Customer_idComboBox.SelectedIndexChanged
        Dim row As OrderDataSet.customerRow

        ' 何も選択されていない場合は、住所と電話番号をクリアする
        If Customer_idComboBox.SelectedItem Is Nothing Then
            txtAddress.Text = ""
            txtPhone.Text = ""
        Else
            ' 選択された顧客のレコードを得る
            row = Customer_idComboBox.SelectedItem.row
            ' 住所と電話番号を表示する
            txtAddress.Text = row.address
            txtPhone.Text = Trim(row.phone)
        End If
    End Sub

    Private Sub BindingNavigatorAddNewItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorAddNewItem.Click
        Dim dv As New DataView, newId As Integer
        ' 入力エラーがあるときは何もしない
        If HandleInputError() = False Then
            Return
        End If
        ' 伝票サブ部分の編集を確定する
        EndEditSub()
        ' order_mainテーブルをDataViewにコピーし、id列の値が大きい順に並べ替える
        dv.Table = OrderDataSet.order_main
        dv.Sort = "id desc"
        If dv.Count = 0 Then
            ' order_mainテーブルにレコードがなければ、新しいレコードのidを1にする
            newId = 1
        Else
            ' order_mainテーブルにレコードがあれば、
            ' 新しいレコードのidを、最大のidの次の番号にする
            newId = dv.Item(0)("id") + 1
        End If
        ' レコードを新規作成する
        Order_mainBindingSource.AddNew()
        ' idのテキストボックスに新しいレコードのidを設定する
        IdTextBox.Text = newId
        ' 日付（date）のDateTimePickerに現在の日時を設定する
        DateDateTimePicker.Value = Now()
    End Sub

    Private Sub Order_subDataGridView_CellValueChanged(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) Handles Order_subDataGridView.CellValueChanged
        Dim dv As New DataView
        ' 商品が選ばれた場合
        If e.ColumnIndex = 2 And e.RowIndex >= 0 Then
            ' 商品テーブルから、選ばれた商品の行を得る
            dv.Table = OrderDataSet.item
            dv.RowFilter = "id = " & Order_subDataGridView.Rows(e.RowIndex).Cells(2).Value
            ' 行が見つかったら、単価列のセルにその商品の単価を設定する
            If dv.Count <> 0 Then
                Order_subDataGridView.Rows(e.RowIndex).Cells(3).Value = dv.Item(0)("price")
            End If
        End If
        ' 個数または単価が入力された場合
        If (e.ColumnIndex = 4 Or e.ColumnIndex = 3) And e.RowIndex >= 0 Then
            ' 各商品の金額と合計金額を求めて表示する
            CalcTotal()
        End If
    End Sub

    Sub CalcTotal()
        Dim price, count, subtotal, total As Integer
        Dim r As Integer
        total = 0
        For r = 0 To Order_subDataGridView.Rows.Count - 1
            ' 各商品の単価／個数／金額を求める
            Try
                price = Val(Order_subDataGridView.Rows(r).Cells(3).Value)
                count = Val(Order_subDataGridView.Rows(r).Cells(4).Value)
                subtotal = price * count
            Catch ex As Exception
                subtotal = 0
            End Try
            ' 金額を表示する
            Order_subDataGridView.Rows(r).Cells(5).Value = subtotal
            ' 合計に金額を足す
            total += subtotal
        Next
        ' 合計を表示する
        txtTotal.Text = total
    End Sub

    Private Sub Order_subDataGridView_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles Order_subDataGridView.Enter
        ' 入力エラーがあるときは何もしない
        If HandleInputError() = False Then
            Return
        End If
    End Sub

    Private Sub BindingNavigatorMoveFirstItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMoveFirstItem.Click
        ' 入力エラーがあるときは何もしない
        If HandleInputError() = False Then
            Return
        End If
        ' 伝票サブ部分の編集を確定する
        EndEditSub()
        ' 先頭のレコードに移動する
        Order_mainBindingSource.MoveFirst()
        ' 合計を計算して表示する
        CalcTotal()
    End Sub

    Private Sub BindingNavigatorMovePreviousItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMovePreviousItem.Click
        ' 入力エラーがあるときは何もしない
        If HandleInputError() = False Then
            Return
        End If
        ' 伝票サブ部分の編集を確定する
        EndEditSub()
        ' 前のレコードに移動する
        Order_mainBindingSource.MovePrevious()
        ' 合計を計算して表示する
        CalcTotal()
    End Sub

    Private Sub BindingNavigatorMoveNextItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMoveNextItem.Click
        ' 入力エラーがあるときは何もしない
        If HandleInputError() = False Then
            Return
        End If
        ' 伝票サブ部分の編集を確定する
        EndEditSub()
        ' 次のレコードに移動する
        Order_mainBindingSource.MoveNext()
        ' 合計を計算して表示する
        CalcTotal()
    End Sub

    Private Sub BindingNavigatorMoveLastItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorMoveLastItem.Click
        ' 入力エラーがあるときは何もしない
        If HandleInputError() = False Then
            Return
        End If
        ' 伝票サブ部分の編集を確定する
        EndEditSub()
        ' 最後のレコードに移動する
        Order_mainBindingSource.MoveLast()
        ' 合計を計算して表示する
        CalcTotal()
    End Sub

    Function HandleInputError() As Boolean
        ' レコードがないときはエラーチェックしない
        If Order_mainBindingSource.Count = 0 Then
            Return True
        End If
        ' 顧客が選択されていない場合はエラーとする
        If Customer_idComboBox.SelectedItem Is Nothing Then
            MsgBox("顧客が選択されていません")
            Customer_idComboBox.Focus()
            Return False
        End If
        ' 伝票メインテーブルの編集を確定する
        Order_mainBindingSource.EndEdit()
        Return True
    End Function

    Sub EndEditSub()
        ' 伝票サブのDataGridViewの編集を確定する
        Try
            Order_subBindingSource.EndEdit()
        Catch ex As Exception
        End Try
    End Sub

    Private Sub Order_subDataGridView_DataError(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewDataErrorEventArgs) Handles Order_subDataGridView.DataError
        ' 伝票サブ部分のエラー処理
        If e.Context = DataGridViewDataErrorContexts.Commit Then
            MsgBox("入力途中で他のレコードに移動しようとしました。")
        End If
    End Sub

    Private Sub BindingNavigatorDeleteItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BindingNavigatorDeleteItem.Click
        ' 削除してよいかどうかを尋ねる
        If MsgBox("この伝票を削除してもよろしいですか", MsgBoxStyle.YesNo + MsgBoxStyle.DefaultButton2) = MsgBoxResult.Yes Then
            ' 「はい」がクリックされた場合は削除する
            Order_mainBindingSource.RemoveCurrent()
        End If
        ' 移動先レコードにあわせて合計を計算して表示する
        CalcTotal()
    End Sub
End Class